@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings FATAL => 'all';
 use parent qw(Exporter);
-use Carp qw(croak);
+use Carp qw(carp croak);
 use PPI::Lexer qw();
 use Scalar::Util qw(looks_like_number);
 
@@ -16,11 +16,11 @@ Struct::Path::PerlStyle - Perl-style syntax frontend for L<Struct::Path|Struct::
 
 =head1 VERSION
 
-Version 0.33
+Version 0.41
 
 =cut
 
-our $VERSION = '0.33';
+our $VERSION = '0.41';
 
 =head1 SYNOPSIS
 
@@ -45,8 +45,8 @@ Examples:
     "{a}{b}[0,1,2,5]"     # 0, 1, 2 and 5 array's items
     "{a}{b}[0..2,5]"      # same, but using ranges
     "{a}{b}[9..0]"        # descending ranges allowed (perl doesn't)
-    "{a}{b}<{c}"          # step back (perl incompatible)
-
+    "{a}{b}<<{c}"         # step back (perl incompatible)
+    "{a}{/regexp/}"       # regexp keys match
 
 =head1 SUBROUTINES
 
@@ -59,11 +59,12 @@ Parse perl-style string to L<Struct::Path|Struct::Path> path
 =cut
 
 our $OPERATORS = {
-    '<' => sub {
+    '<<' => sub { # step back
         pop @{$_[0]};
         pop @{$_[1]}
     },
 };
+$OPERATORS->{'<'} = $OPERATORS->{'<<'}; # deprecation cycle backward compatibility (since 0.41)
 
 sub ps_parse($) {
     my $path = shift;
@@ -82,17 +83,20 @@ sub ps_parse($) {
                 $item->isa('PPI::Structure::Block')) {
                 push @{$out}, {};
                 for my $t (map { $_->elements } $item->children) {
-                    my $key;
+                    my $tmp;
                     if ($t->isa('PPI::Token::Word') or $t->isa('PPI::Token::Number')) {
-                        $key = $t->content;
+                        $tmp->{keys} = $t->content;
                     } elsif ($t->isa('PPI::Token::Operator') and $t->content eq ',') {
                         next;
                     } elsif ($t->isa('PPI::Token::Quote')) {
-                        $key = substr(substr($t->content, 1), 0, -1);
+                        $tmp->{keys} = substr(substr($t->content, 1), 0, -1);
+                    } elsif ($t->isa('PPI::Token::Regexp::Match')) {
+                        $tmp->{regs} = substr(substr($t->content, 1), 0, -1); # get rid of slashes
+                        $tmp->{regs} = qr($tmp->{regs});
                     } else {
                         croak "Unsupported thing '" . $t->content . "' in hash key specification (step #$sc)";
                     }
-                    push @{$out->[-1]->{keys}}, $key;
+                    map { push @{$out->[-1]->{$_}}, delete $tmp->{$_} } keys %{$tmp};
                 }
             } elsif ($item->isa('PPI::Structure::Constructor')) { # PPI::Structure::Constructor is hash/array constructir only
                 push @{$out}, [];
@@ -120,6 +124,8 @@ sub ps_parse($) {
                 }
                 croak "Unfinished range secified (step #$sc)" if ($is_range);
             } elsif ($item->isa('PPI::Token::Operator') and exists $OPERATORS->{$item->content}) {
+                carp "Operator '<' is deprecated and will be reused in future releases for other" .
+                    " purposes. Use '<<' instead." if ($item->content eq '<');
                 push @{$out}, $OPERATORS->{$item->content};
             } else {
                 croak "Unsupported thing '" . $item->content . "' in the path (step #$sc)" ;
